// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/datacatalog/lineage/v1/lineage.proto

#include "google/cloud/datacatalog/lineage/v1/internal/lineage_metadata_decorator.h"
#include "google/cloud/common_options.h"
#include "google/cloud/internal/absl_str_cat_quiet.h"
#include "google/cloud/internal/api_client_header.h"
#include "google/cloud/internal/url_encode.h"
#include "google/cloud/status_or.h"
#include <google/cloud/datacatalog/lineage/v1/lineage.grpc.pb.h>
#include <memory>
#include <utility>

namespace google {
namespace cloud {
namespace datacatalog_lineage_v1_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

LineageMetadata::LineageMetadata(
    std::shared_ptr<LineageStub> child,
    std::multimap<std::string, std::string> fixed_metadata,
    std::string api_client_header)
    : child_(std::move(child)),
      fixed_metadata_(std::move(fixed_metadata)),
      api_client_header_(
          api_client_header.empty()
              ? google::cloud::internal::GeneratedLibClientHeader()
              : std::move(api_client_header)) {}

StatusOr<google::cloud::datacatalog::lineage::v1::Process>
LineageMetadata::CreateProcess(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::CreateProcessRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->CreateProcess(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::Process>
LineageMetadata::UpdateProcess(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::UpdateProcessRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("process.name=",
                           internal::UrlEncode(request.process().name())));
  return child_->UpdateProcess(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::Process>
LineageMetadata::GetProcess(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::GetProcessRequest const& request) {
  SetMetadata(context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetProcess(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::ListProcessesResponse>
LineageMetadata::ListProcesses(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::ListProcessesRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->ListProcesses(context, request);
}

future<StatusOr<google::longrunning::Operation>>
LineageMetadata::AsyncDeleteProcess(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::datacatalog::lineage::v1::DeleteProcessRequest const&
        request) {
  SetMetadata(*context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncDeleteProcess(cq, std::move(context), request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::Run>
LineageMetadata::CreateRun(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::CreateRunRequest const& request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->CreateRun(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::Run>
LineageMetadata::UpdateRun(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::UpdateRunRequest const& request) {
  SetMetadata(context, absl::StrCat("run.name=",
                                    internal::UrlEncode(request.run().name())));
  return child_->UpdateRun(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::Run> LineageMetadata::GetRun(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::GetRunRequest const& request) {
  SetMetadata(context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetRun(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::ListRunsResponse>
LineageMetadata::ListRuns(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::ListRunsRequest const& request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->ListRuns(context, request);
}

future<StatusOr<google::longrunning::Operation>>
LineageMetadata::AsyncDeleteRun(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::datacatalog::lineage::v1::DeleteRunRequest const& request) {
  SetMetadata(*context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncDeleteRun(cq, std::move(context), request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::LineageEvent>
LineageMetadata::CreateLineageEvent(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::CreateLineageEventRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->CreateLineageEvent(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::LineageEvent>
LineageMetadata::GetLineageEvent(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::GetLineageEventRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetLineageEvent(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::ListLineageEventsResponse>
LineageMetadata::ListLineageEvents(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::ListLineageEventsRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->ListLineageEvents(context, request);
}

Status LineageMetadata::DeleteLineageEvent(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::DeleteLineageEventRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->DeleteLineageEvent(context, request);
}

StatusOr<google::cloud::datacatalog::lineage::v1::SearchLinksResponse>
LineageMetadata::SearchLinks(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::SearchLinksRequest const&
        request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->SearchLinks(context, request);
}

StatusOr<
    google::cloud::datacatalog::lineage::v1::BatchSearchLinkProcessesResponse>
LineageMetadata::BatchSearchLinkProcesses(
    grpc::ClientContext& context,
    google::cloud::datacatalog::lineage::v1::
        BatchSearchLinkProcessesRequest const& request) {
  SetMetadata(context,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->BatchSearchLinkProcesses(context, request);
}

future<StatusOr<google::longrunning::Operation>>
LineageMetadata::AsyncGetOperation(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::longrunning::GetOperationRequest const& request) {
  SetMetadata(*context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncGetOperation(cq, std::move(context), request);
}

future<Status> LineageMetadata::AsyncCancelOperation(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::longrunning::CancelOperationRequest const& request) {
  SetMetadata(*context,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncCancelOperation(cq, std::move(context), request);
}

void LineageMetadata::SetMetadata(grpc::ClientContext& context,
                                  std::string const& request_params) {
  context.AddMetadata("x-goog-request-params", request_params);
  SetMetadata(context);
}

void LineageMetadata::SetMetadata(grpc::ClientContext& context) {
  for (auto const& kv : fixed_metadata_) {
    context.AddMetadata(kv.first, kv.second);
  }
  context.AddMetadata("x-goog-api-client", api_client_header_);
  auto const& options = internal::CurrentOptions();
  if (options.has<UserProjectOption>()) {
    context.AddMetadata("x-goog-user-project",
                        options.get<UserProjectOption>());
  }
  auto const& authority = options.get<AuthorityOption>();
  if (!authority.empty()) context.set_authority(authority);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace datacatalog_lineage_v1_internal
}  // namespace cloud
}  // namespace google
