// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dataproc/v1/batches.proto

#include "google/cloud/dataproc/v1/internal/batch_controller_connection_impl.h"
#include "google/cloud/dataproc/v1/internal/batch_controller_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/async_long_running_operation.h"
#include "google/cloud/internal/pagination_range.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>

namespace google {
namespace cloud {
namespace dataproc_v1_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
namespace {

std::unique_ptr<dataproc_v1::BatchControllerRetryPolicy> retry_policy(
    Options const& options) {
  return options.get<dataproc_v1::BatchControllerRetryPolicyOption>()->clone();
}

std::unique_ptr<BackoffPolicy> backoff_policy(Options const& options) {
  return options.get<dataproc_v1::BatchControllerBackoffPolicyOption>()
      ->clone();
}

std::unique_ptr<dataproc_v1::BatchControllerConnectionIdempotencyPolicy>
idempotency_policy(Options const& options) {
  return options
      .get<dataproc_v1::BatchControllerConnectionIdempotencyPolicyOption>()
      ->clone();
}

std::unique_ptr<PollingPolicy> polling_policy(Options const& options) {
  return options.get<dataproc_v1::BatchControllerPollingPolicyOption>()
      ->clone();
}

}  // namespace

BatchControllerConnectionImpl::BatchControllerConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<dataproc_v1_internal::BatchControllerStub> stub,
    Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(std::move(options),
                                      BatchControllerConnection::options())) {}

future<StatusOr<google::cloud::dataproc::v1::Batch>>
BatchControllerConnectionImpl::CreateBatch(
    google::cloud::dataproc::v1::CreateBatchRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::AsyncLongRunningOperation<
      google::cloud::dataproc::v1::Batch>(
      background_->cq(), request,
      [stub = stub_](
          google::cloud::CompletionQueue& cq,
          std::shared_ptr<grpc::ClientContext> context,
          google::cloud::dataproc::v1::CreateBatchRequest const& request) {
        return stub->AsyncCreateBatch(cq, std::move(context), request);
      },
      [stub = stub_](google::cloud::CompletionQueue& cq,
                     std::shared_ptr<grpc::ClientContext> context,
                     google::longrunning::GetOperationRequest const& request) {
        return stub->AsyncGetOperation(cq, std::move(context), request);
      },
      [stub = stub_](
          google::cloud::CompletionQueue& cq,
          std::shared_ptr<grpc::ClientContext> context,
          google::longrunning::CancelOperationRequest const& request) {
        return stub->AsyncCancelOperation(cq, std::move(context), request);
      },
      &google::cloud::internal::ExtractLongRunningResultResponse<
          google::cloud::dataproc::v1::Batch>,
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->CreateBatch(request),
      polling_policy(*current), __func__);
}

StatusOr<google::cloud::dataproc::v1::Batch>
BatchControllerConnectionImpl::GetBatch(
    google::cloud::dataproc::v1::GetBatchRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->GetBatch(request),
      [this](grpc::ClientContext& context,
             google::cloud::dataproc::v1::GetBatchRequest const& request) {
        return stub_->GetBatch(context, request);
      },
      request, __func__);
}

StreamRange<google::cloud::dataproc::v1::Batch>
BatchControllerConnectionImpl::ListBatches(
    google::cloud::dataproc::v1::ListBatchesRequest request) {
  request.clear_page_token();
  auto current = google::cloud::internal::SaveCurrentOptions();
  auto idempotency = idempotency_policy(*current)->ListBatches(request);
  char const* function_name = __func__;
  return google::cloud::internal::MakePaginationRange<
      StreamRange<google::cloud::dataproc::v1::Batch>>(
      std::move(request),
      [idempotency, function_name, stub = stub_,
       retry = std::shared_ptr<dataproc_v1::BatchControllerRetryPolicy>(
           retry_policy(*current)),
       backoff = std::shared_ptr<BackoffPolicy>(backoff_policy(*current))](
          google::cloud::dataproc::v1::ListBatchesRequest const& r) {
        return google::cloud::internal::RetryLoop(
            retry->clone(), backoff->clone(), idempotency,
            [stub](grpc::ClientContext& context,
                   google::cloud::dataproc::v1::ListBatchesRequest const&
                       request) { return stub->ListBatches(context, request); },
            r, function_name);
      },
      [](google::cloud::dataproc::v1::ListBatchesResponse r) {
        std::vector<google::cloud::dataproc::v1::Batch> result(
            r.batches().size());
        auto& messages = *r.mutable_batches();
        std::move(messages.begin(), messages.end(), result.begin());
        return result;
      });
}

Status BatchControllerConnectionImpl::DeleteBatch(
    google::cloud::dataproc::v1::DeleteBatchRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->DeleteBatch(request),
      [this](grpc::ClientContext& context,
             google::cloud::dataproc::v1::DeleteBatchRequest const& request) {
        return stub_->DeleteBatch(context, request);
      },
      request, __func__);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dataproc_v1_internal
}  // namespace cloud
}  // namespace google
